# coding=utf-8

import platform
import textwrap
import unittest

import pytest

from conans.client import tools
from conans.paths import CONANFILE
from conans.test.assets.cmake import gen_cmakelists
from conans.test.assets.cpp_test_files import cpp_hello_conan_files
from conans.test.assets.sources import gen_function_cpp, gen_function_h
from conans.test.utils.tools import TestClient, TestServer
from conans.util.runners import check_output_runner


@pytest.mark.tool_cmake
class RunEnvironmentTest(unittest.TestCase):

    def test_run_environment(self):
        client = TestClient()
        files = cpp_hello_conan_files("Hello0", "0.1")
        files[CONANFILE] = files[CONANFILE].replace(
            'self.copy(pattern="*.so", dst="lib", keep_path=False)',
            '''self.copy(pattern="*.so", dst="lib", keep_path=False)
        self.copy(pattern="*say_hello*", dst="bin", keep_path=False)''')
        client.save(files)
        client.run("export . lasote/stable")

        reuse = textwrap.dedent("""
            from conans import ConanFile, RunEnvironment, tools

            class HelloConan(ConanFile):
                name = "Reuse"
                version = "0.1"
                build_policy = "missing"
                requires = "Hello0/0.1@lasote/stable"

                def build(self):
                    run_env = RunEnvironment(self)
                    with tools.environment_append(run_env.vars):
                        self.run("say_hello")
        """)

        client.save({"conanfile.py": reuse}, clean_first=True)
        client.run("install . --build missing")
        client.run("build .")
        self.assertIn("Hello Hello0", client.out)


@pytest.mark.tool_cmake
class RunEnvironmentSharedTest(unittest.TestCase):

    def setUp(self):
        self.servers = {"default": TestServer()}
        client = TestClient(servers=self.servers, users={"default": [("lasote", "mypass")]})
        cmake = gen_cmakelists(libname="hello", libsources=["hello.cpp"], libtype="SHARED",
                               appname="say_hello", appsources=["main.cpp"])
        hello_h = gen_function_h(name="hello")
        hello_cpp = gen_function_cpp(name="hello", msg="Hello Tool!", includes=["hello"])
        main = gen_function_cpp(name="main", includes=["hello"], calls=["hello"])

        conanfile = textwrap.dedent("""
            from conans import ConanFile, CMake
            class Pkg(ConanFile):
                settings = "os", "compiler", "build_type", "arch"
                exports_sources = "*"

                def build(self):
                    cmake = CMake(self)
                    cmake.configure()
                    cmake.build()

                def package(self):
                    self.copy("*say_hello.exe", dst="bin", keep_path=False)
                    self.copy("*say_hello", dst="bin", keep_path=False)
                    self.copy(pattern="*.dll", dst="bin", keep_path=False)
                    self.copy(pattern="*.dylib", dst="lib", keep_path=False)
                    self.copy(pattern="*.so", dst="lib", keep_path=False)
        """)

        client.save({"conanfile.py": conanfile,
                     "CMakeLists.txt": cmake,
                     "main.cpp": main,
                     "hello.cpp": hello_cpp,
                     "hello.h": hello_h})
        client.run("create . Pkg/0.1@lasote/testing")
        client.run("upload Pkg* --all --confirm")
        client.run('remove "*" -f')
        client.run("search")
        self.assertIn("There are no packages", client.out)

    def test_run_with_run_environment(self):
        client = TestClient(servers=self.servers, users={"default": [("lasote", "mypass")]})

        reuse = textwrap.dedent("""
            from conans import ConanFile
            class HelloConan(ConanFile):
                requires = "Pkg/0.1@lasote/testing"

                def build(self):
                    self.run("say_hello", run_environment=True)
        """)

        client.save({"conanfile.py": reuse}, clean_first=True)
        client.run("install .")
        client.run("build .")
        self.assertIn("Hello Tool!", client.out)

    @pytest.mark.skipif(platform.system() != "Darwin", reason="Check SIP protection shell=False")
    def test_command_as_list(self):
        client = TestClient(servers=self.servers, users={"default": [("lasote", "mypass")]})

        reuse = textwrap.dedent("""
            from conans import ConanFile
            class HelloConan(ConanFile):
                requires = "Pkg/0.1@lasote/testing"
                options = {"cmd_list": [True, False]}
                default_options = {"cmd_list": False}

                def build(self):
                    if self.options.cmd_list:
                        self.run(["say_hello",], run_environment=True)
                    else:
                        self.run("say_hello", run_environment=True)
        """)

        client.save({"conanfile.py": reuse}, clean_first=True)
        client.run("config set log.print_run_commands=1")

        # Using a string, a new shell is expanded, DYLD_... paths has to be informed
        client.run("install . -o cmd_list=False")
        client.run("build .")
        self.assertIn("> DYLD_LIBRARY_PATH=", client.out)
        self.assertIn("Hello Tool!", client.out)

        # Using a list, no new shell, DYLD_... are already in the environment
        client.run("install . -o cmd_list=True")
        client.run("build .")
        self.assertNotIn("DYLD_LIBRARY_PATH", client.out)
        self.assertIn("Hello Tool!", client.out)

    @pytest.mark.skipif(platform.system() == "Darwin", reason="SIP protection (read comment)")
    def test_with_tools_run_environment(self):
        # This test is excluded from OSX, because of the SIP protection. CMake helper will
        # launch a subprocess with shell=True, which CLEANS the DYLD_LIBRARY_PATH. Injecting its
        # value via run_environment=True doesn't work, because it prepends its value to:
        # command = "cd [folder] && cmake [cmd]" =>
        #                 "DYLD_LIBRARY_PATH=[path] cd [folder] && cmake [cmd]"
        # and then only applies to the change directory "cd"
        # If CMake binary is in user folder, it is not under SIP, and it can work. For cmake
        # installed in system folders, then no possible form of "DYLD_LIBRARY_PATH=[folders] cmake"
        # can work
        client = TestClient(servers=self.servers, users={"default": [("lasote", "mypass")]})

        reuse = textwrap.dedent("""
            from conans import ConanFile, CMake, tools
            class HelloConan(ConanFile):
                exports = "CMakeLists.txt"
                requires = "Pkg/0.1@lasote/testing"

                def build(self):
                    with tools.run_environment(self):
                        cmake = CMake(self)
                        cmake.configure()
                        cmake.build()
        """)

        cmake = textwrap.dedent("""
            set(CMAKE_CXX_COMPILER_WORKS 1)
            set(CMAKE_CXX_ABI_COMPILED 1)
            project(MyHello CXX)
            cmake_minimum_required(VERSION 2.8.12)
            execute_process(COMMAND say_hello)
        """)

        client.save({"conanfile.py": reuse,
                     "CMakeLists.txt": cmake}, clean_first=True)
        client.run("install .")
        client.run("build .")
        self.assertIn("Hello Tool!", client.out)

    def test_virtualrunenv(self):
        client = TestClient(servers=self.servers, users={"default": [("lasote", "mypass")]})
        client.run("install Pkg/0.1@lasote/testing -g virtualrunenv")

        with tools.chdir(client.current_folder):
            if platform.system() == "Windows":
                command = "activate_run.bat && say_hello"
            else:
                # It is not necessary to use the DYLD_LIBRARY_PATH in OSX because the activate_run.sh
                # will work perfectly. It is inside bash, so the loader will use DYLD_LIBRARY_PATH
                # values. It also works in command line with export DYLD_LIBRARY_PATH=[path] and then
                # running, or in the same line "$ DYLD_LIBRARY_PATH=[path] say_hello"
                command = "bash -c 'source activate_run.sh && say_hello'"

            output = check_output_runner(command)
            self.assertIn("Hello Tool!", output)



import unittest

import six

from conans.errors import ConanException
from conans.model.ref import ConanFileReference, ConanName, InvalidNameException, PackageReference, \
    check_valid_ref, get_reference_fields


class RefTest(unittest.TestCase):
    def test_basic(self):
        ref = ConanFileReference.loads("opencv/2.4.10@lasote/testing")
        self.assertEqual(ref.name, "opencv")
        self.assertEqual(ref.version, "2.4.10")
        self.assertEqual(ref.user, "lasote")
        self.assertEqual(ref.channel, "testing")
        self.assertEqual(ref.revision, None)
        self.assertEqual(str(ref), "opencv/2.4.10@lasote/testing")

        ref = ConanFileReference.loads("opencv_lite/2.4.10@phil_lewis/testing")
        self.assertEqual(ref.name, "opencv_lite")
        self.assertEqual(ref.version, "2.4.10")
        self.assertEqual(ref.user, "phil_lewis")
        self.assertEqual(ref.channel, "testing")
        self.assertEqual(ref.revision, None)
        self.assertEqual(str(ref), "opencv_lite/2.4.10@phil_lewis/testing")

        ref = ConanFileReference.loads("opencv/2.4.10@3rd-party/testing")
        self.assertEqual(ref.name, "opencv")
        self.assertEqual(ref.version, "2.4.10")
        self.assertEqual(ref.user, "3rd-party")
        self.assertEqual(ref.channel, "testing")
        self.assertEqual(ref.revision, None)
        self.assertEqual(str(ref), "opencv/2.4.10@3rd-party/testing")

        ref = ConanFileReference.loads("opencv/2.4.10@3rd-party/testing#rev1")
        self.assertEqual(ref.revision, "rev1")

    def test_errors(self):
        self.assertRaises(ConanException, ConanFileReference.loads, "")
        self.assertIsNone(ConanFileReference.loads("opencv/2.4.10@", validate=False).channel)
        self.assertIsNone(ConanFileReference.loads("opencv/2.4.10@", validate=False).user)
        self.assertRaises(ConanException, ConanFileReference.loads, "opencv/2.4.10@lasote")
        self.assertRaises(ConanException, ConanFileReference.loads, "opencv??/2.4.10@laso/testing")
        self.assertRaises(ConanException, ConanFileReference.loads, "opencv/2.4.10 @ laso/testing")
        self.assertRaises(ConanException, ConanFileReference.loads, "o/2.4.10@laso/testing")
        self.assertRaises(ConanException, ConanFileReference.loads, ".opencv/2.4.10@lasote/testing")
        self.assertRaises(ConanException, ConanFileReference.loads, "o/2.4.10 @ lasote/testing")
        self.assertRaises(ConanException, ConanFileReference.loads, "lib/1.0@user&surname/channel")
        self.assertRaises(ConanException, ConanFileReference.loads,
                          "opencv%s/2.4.10@laso/testing" % "A" * 40)
        self.assertRaises(ConanException, ConanFileReference.loads,
                          "opencv/2.4.10%s@laso/testing" % "A" * 40)
        self.assertRaises(ConanException, ConanFileReference.loads,
                          "opencv/2.4.10@laso%s/testing" % "A" * 40)
        self.assertRaises(ConanException, ConanFileReference.loads,
                          "opencv/2.4.10@laso/testing%s" % "A" * 40)

        self.assertRaises(ConanException, ConanFileReference.loads, "opencv/2.4.10/laso/testing")
        self.assertRaises(ConanException, ConanFileReference.loads, "opencv/2.4.10/laso/test#1")
        self.assertRaises(ConanException, ConanFileReference.loads, "opencv@2.4.10/laso/test")
        self.assertRaises(ConanException, ConanFileReference.loads, "opencv/2.4.10/laso@test")

    def test_revisions(self):
        ref = ConanFileReference.loads("opencv/2.4.10@lasote/testing#23")
        self.assertEqual(ref.channel, "testing")
        self.assertEqual(ref.revision, "23")

        ref = ConanFileReference.loads("opencv/2.4.10#23")
        self.assertIsNone(ref.channel)
        self.assertIsNone(ref.user)
        self.assertEqual(ref.name, "opencv")
        self.assertEqual(ref.version, "2.4.10")
        self.assertEqual(ref.revision, "23")

        ref = ConanFileReference("opencv", "2.3", "lasote", "testing", "34")
        self.assertEqual(ref.revision, "34")

        pref = PackageReference.loads("opencv/2.4.10@lasote/testing#23:123123123#989")
        self.assertEqual(pref.revision, "989")
        self.assertEqual(pref.ref.revision, "23")

        pref = PackageReference(ref, "123123123#989")
        self.assertEqual(pref.ref.revision, "34")

    def test_equal(self):
        ref = ConanFileReference.loads("opencv/2.4.10@lasote/testing#23")
        ref2 = ConanFileReference.loads("opencv/2.4.10@lasote/testing#232")
        self.assertFalse(ref == ref2)
        self.assertTrue(ref != ref2)

        ref = ConanFileReference.loads("opencv/2.4.10@lasote/testing")
        ref2 = ConanFileReference.loads("opencv/2.4.10@lasote/testing#232")

        self.assertFalse(ref == ref2)
        self.assertTrue(ref != ref2)
        self.assertTrue(ref2 != ref)

        ref = ConanFileReference.loads("opencv/2.4.10@lasote/testing")
        ref2 = ConanFileReference.loads("opencv/2.4.10@lasote/testing")

        self.assertTrue(ref == ref2)
        self.assertFalse(ref != ref2)

        ref = ConanFileReference.loads("opencv/2.4.10@lasote/testing#23")
        ref2 = ConanFileReference.loads("opencv/2.4.10@lasote/testing#23")
        self.assertTrue(ref == ref2)
        self.assertFalse(ref != ref2)


class ConanNameTestCase(unittest.TestCase):

    def _check_invalid_format(self, value, *args):
        with six.assertRaisesRegex(self, InvalidNameException, "Valid names"):
            ConanName.validate_name(value, *args)

    def _check_invalid_version(self, name, version):
        with six.assertRaisesRegex(self, InvalidNameException, "invalid version number"):
            ConanName.validate_version(version, name)

    def _check_invalid_type(self, value):
        with six.assertRaisesRegex(self, InvalidNameException, "is not a string"):
            ConanName.validate_name(value)

    def test_validate_name(self):
        self.assertIsNone(ConanName.validate_name("string.dot.under-score.123"))
        self.assertIsNone(ConanName.validate_name("_underscore+123"))
        self.assertIsNone(ConanName.validate_name("*"))
        self.assertIsNone(ConanName.validate_name("a" * ConanName._min_chars))
        self.assertIsNone(ConanName.validate_name("a" * ConanName._max_chars))
        self.assertIsNone(ConanName.validate_name("a" * 50))  # Regression test

    def test_validate_name_invalid_format(self):
        self._check_invalid_format("-no.dash.start")
        self._check_invalid_format("a" * (ConanName._min_chars - 1))
        self._check_invalid_format("a" * (ConanName._max_chars + 1))

    def test_validate_name_invalid_type(self):
        self._check_invalid_type(123.34)
        self._check_invalid_type(("item1", "item2",))

    def test_validate_name_version(self):
        self.assertIsNone(ConanName.validate_version("name", "[vvvv]"))

    def test_validate_name_version_invalid(self):
        self._check_invalid_version("name", "[no.close.bracket")
        self._check_invalid_version("name", "no.open.bracket]")


class CheckValidRefTest(unittest.TestCase):

    def test_string(self):
        self.assertTrue(check_valid_ref("package/1.0@user/channel"))
        self.assertTrue(check_valid_ref("package/1.0@user/channel"))
        self.assertTrue(check_valid_ref("package/[*]@user/channel"))
        self.assertTrue(check_valid_ref("package/[>1.0]@user/channel"))
        self.assertTrue(check_valid_ref("package/[1.*]@user/channel"))

        # Patterns are invalid
        self.assertFalse(check_valid_ref("package/*@user/channel"))
        self.assertFalse(check_valid_ref("package/1.0@user/*"))
        self.assertFalse(check_valid_ref("package/1.0@user/chan*"))
        self.assertFalse(check_valid_ref("package/[>1.0]@user/chan*"))
        self.assertFalse(check_valid_ref("*/1.0@user/channel"))
        self.assertFalse(check_valid_ref("package*/1.0@user/channel"))

        # * pattern is valid in non stric_mode
        self.assertTrue(check_valid_ref("package/*@user/channel", strict_mode=False))
        self.assertTrue(check_valid_ref("package/*@user/*", strict_mode=False))

        # But other patterns are not valid in non stric_mode
        self.assertFalse(check_valid_ref("package/1.0@user/chan*", strict_mode=False))

    def test_incomplete_refs(self):
        self.assertTrue(check_valid_ref("package/1.0", strict_mode=False))
        self.assertFalse(check_valid_ref("package/1.0"))
        self.assertFalse(check_valid_ref("package/1.0@user"))
        self.assertFalse(check_valid_ref("package/1.0@/channel"))
        self.assertFalse(check_valid_ref("lib@#rev"))


class GetReferenceFieldsTest(unittest.TestCase):

    def test_fields_complete(self):

        # No matter if we say we allow partial references for "user/channel", if we
        # provide this patterns everything is parsed correctly
        for user_channel_input in [True, False]:
            tmp = get_reference_fields("lib/1.0@user", user_channel_input=user_channel_input)
            self.assertEqual(tmp, ("lib", "1.0", "user", None, None))

            tmp = get_reference_fields("lib/1.0@/channel", user_channel_input=user_channel_input)
            self.assertEqual(tmp, ("lib", "1.0", None, "channel", None))

            # FIXME: 2.0 in this case lib is considered the version, weird.
            tmp = get_reference_fields("lib@#rev", user_channel_input=user_channel_input)
            self.assertEqual(tmp, (None, "lib", None, None, "rev"))

            # FIXME: 2.0 in this case lib is considered the version, weird.
            tmp = get_reference_fields("lib@/channel#rev", user_channel_input=user_channel_input)
            self.assertEqual(tmp, (None, "lib", None, "channel", "rev"))

            tmp = get_reference_fields("/1.0@user/#rev", user_channel_input=user_channel_input)
            self.assertEqual(tmp, (None, "1.0", "user", None, "rev"))

            tmp = get_reference_fields("/@/#", user_channel_input=user_channel_input)
            self.assertEqual(tmp, (None, None, None, None, None))

            tmp = get_reference_fields("lib/1.0@/", user_channel_input=user_channel_input)
            self.assertEqual(tmp, ("lib", "1.0", None, None, None))

            tmp = get_reference_fields("lib/1.0@", user_channel_input=user_channel_input)
            self.assertEqual(tmp, ("lib", "1.0", None, None, None))

            tmp = get_reference_fields("lib/@", user_channel_input=user_channel_input)
            self.assertEqual(tmp, ("lib", None, None, None, None))

            tmp = get_reference_fields("/@", user_channel_input=user_channel_input)
            self.assertEqual(tmp, (None, None, None, None, None))

            tmp = get_reference_fields("@", user_channel_input=user_channel_input)
            self.assertEqual(tmp, (None, None, None, None, None))

            tmp = get_reference_fields("lib/1.0@user/channel#rev",
                                       user_channel_input=user_channel_input)
            self.assertEqual(tmp, ("lib", "1.0", "user", "channel", "rev"))

            # FIXME: 2.0 in this case lib is considered the version, weird.
            tmp = get_reference_fields("lib@user/channel", user_channel_input=user_channel_input)
            self.assertEqual(tmp, (None, "lib", "user", "channel", None))

            tmp = get_reference_fields("/@/#", user_channel_input=user_channel_input)
            self.assertEqual(tmp, (None, None, None, None, None))

    def test_only_user_channel(self):
        tmp = get_reference_fields("user/channel", user_channel_input=True)
        self.assertEqual(tmp, (None, None, "user", "channel", None))

        tmp = get_reference_fields("user", user_channel_input=True)
        self.assertEqual(tmp, (None, None, "user", None, None))

        tmp = get_reference_fields("/channel", user_channel_input=True)
        self.assertEqual(tmp, (None, None, None, "channel", None))

        ref_pattern = ConanFileReference.loads("package/*@user/channel")
        self.assertFalse(check_valid_ref(ref_pattern, strict_mode=False))


class CompatiblePrefTest(unittest.TestCase):

    def test_compatible(self):

        def ok(pref1, pref2):
            pref1 = PackageReference.loads(pref1)
            pref2 = PackageReference.loads(pref2)
            return pref1.is_compatible_with(pref2)

        # Same ref is ok
        self.assertTrue(ok("package/1.0@user/channel#RREV1:packageid1#PREV1",
                           "package/1.0@user/channel#RREV1:packageid1#PREV1"))

        # Change PREV is not ok
        self.assertFalse(ok("package/1.0@user/channel#RREV1:packageid1#PREV1",
                            "package/1.0@user/channel#RREV1:packageid1#PREV2"))

        # Different ref is not ok
        self.assertFalse(ok("packageA/1.0@user/channel#RREV1:packageid1#PREV1",
                            "packageB/1.0@user/channel#RREV1:packageid1#PREV1"))

        # Different ref is not ok
        self.assertFalse(ok("packageA/1.0@user/channel#RREV1:packageid1",
                            "packageB/1.0@user/channel#RREV1:packageid1#PREV1"))

        # Different package_id is not ok
        self.assertFalse(ok("packageA/1.0@user/channel#RREV1:packageid1",
                            "packageA/1.0@user/channel#RREV1:packageid2#PREV1"))

        # Completed PREV is ok
        self.assertTrue(ok("packageA/1.0@user/channel#RREV1:packageid1",
                           "packageA/1.0@user/channel#RREV1:packageid1#PREV1"))

        # But only in order, the second ref cannot remove PREV
        self.assertFalse(ok("packageA/1.0@user/channel#RREV1:packageid1#PREV1",
                            "packageA/1.0@user/channel#RREV1:packageid1"))

        # Completing RREV is also OK
        self.assertTrue(ok("packageA/1.0@user/channel:packageid1",
                           "packageA/1.0@user/channel#RREV1:packageid1"))

        # Completing RREV and PREV is also OK
        self.assertTrue(ok("packageA/1.0@user/channel:packageid1",
                           "packageA/1.0@user/channel#RREV:packageid1#PREV"))



# coding=utf-8

import unittest
from collections import namedtuple

from conans.model.conan_file import create_requirements

MockConanFile = namedtuple("MockConanFile", ["requires", ])


class TupleTest(unittest.TestCase):

    def test_implicit_tuple(self):
        requires = "req/1.0@user/version", "req2/1.0@user/version"
        self.assertEqual(type(requires), tuple)
        conanfile = MockConanFile(requires=requires)
        r = create_requirements(conanfile)
        self.assertListEqual(list(r.keys()), ["req", "req2"])

    def test_tuple(self):
        requires = ("req/1.0@user/version", "req2/1.0@user/version")
        self.assertEqual(type(requires), tuple)
        conanfile = MockConanFile(requires=requires)
        r = create_requirements(conanfile)
        self.assertListEqual(list(r.keys()), ["req", "req2"])

    def test_config_tuple(self):
        requires = (("req/1.0@user/version", "private"), )
        self.assertEqual(type(requires), tuple)
        conanfile = MockConanFile(requires=requires)
        r = create_requirements(conanfile)
        self.assertListEqual(list(r.keys()), ["req", ])


class ListTest(unittest.TestCase):

    def test_list(self):
        requires = ["req/1.0@user/version", "req2/1.0@user/version"]
        self.assertEqual(type(requires), list)
        conanfile = MockConanFile(requires=requires)
        r = create_requirements(conanfile)
        self.assertListEqual(list(r.keys()), ["req", "req2"])

    def test_config_list(self):
        requires = [("req/1.0@user/version", "private"), ]
        self.assertEqual(type(requires), list)
        conanfile = MockConanFile(requires=requires)
        r = create_requirements(conanfile)
        self.assertListEqual(list(r.keys()), ["req", ])



# coding=utf-8

import unittest
import os
import platform

import pytest

from conans.migrations import Migrator
from conans.test.utils.mocks import TestBufferConanOutput
from conans.test.utils.test_files import temp_folder
from conans.errors import ConanMigrationError


class FakeMigrator(Migrator):

    def __init__(self, cache_folder, current_version, out):
        self.cache_folder = cache_folder
        super(FakeMigrator, self).__init__(cache_folder, current_version, out)

    def _make_migrations(self, old_version):
        pass


class MigratorPermissionTest(unittest.TestCase):

    @pytest.mark.skipif(platform.system() == "Windows", reason="Can't apply chmod on Windows")
    def test_invalid_permission(self):
        out = TestBufferConanOutput()
        conf_path = temp_folder(False)
        os.chmod(conf_path, 0o444)
        conf_path = os.path.join(conf_path, "foo")
        migrator = FakeMigrator(conf_path, "latest", out)
        with self.assertRaises(ConanMigrationError) as error:
            migrator.migrate()
        self.assertEqual("Can't write version file in '{0}/version.txt': The folder {0} does not "
                         "exist and could not be created (Permission denied).".format(conf_path),
                         str(error.exception))



import os
import textwrap

import pytest

from conans.test.utils.tools import TestClient


@pytest.mark.parametrize(
    "arch,exp_platform",
    [
        ("x86", "Win32"),
        ("x86_64", "x64"),
        ("armv7", "ARM"),
        ("armv8", "ARM64"),
    ],
)
def test_msbuilddeps_maps_architecture_to_platform(arch, exp_platform):
    client = TestClient(path_with_spaces=False)
    client.run("new hello/0.1 --template=msbuild_lib")
    client.run(f"install . -g MSBuildDeps -s arch={arch} -pr:b=default -if=install")
    toolchain = client.load(os.path.join("conan", "conantoolchain.props"))
    expected_import = f"""<Import Condition="'$(Configuration)' == 'Release' And '$(Platform)' == '{exp_platform}'" Project="conantoolchain_release_{exp_platform.lower()}.props"/>"""
    assert expected_import in toolchain


def test_msbuilddeps_format_names():
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            name = "pkg"
            version = "1.0"

            def package_info(self):
                self.cpp_info.components["libmpdecimal++"].libs = ["libmp++"]
        """)
    c.save({"conanfile.py": conanfile})
    c.run("create . -s arch=x86_64")
    # Issue: https://github.com/conan-io/conan/issues/11822
    c.run("install pkg/1.0@ -g MSBuildDeps -s build_type=Release -s arch=x86_64")
    # Checking that MSBuildDeps builds correctly the XML file
    props = c.load("conan_pkg_libmpdecimal__.props")
    assert "<conan_pkg_libmpdecimal___props_imported>" in props
    props = c.load("conan_pkg_libmpdecimal___release_x64.props")
    assert "Conanpkg_libmpdecimal__IncludeDirectories" in props
    props = c.load("conan_pkg_libmpdecimal___vars_release_x64.props")
    assert "<Conanpkg_libmpdecimal__RootFolder>" in props



import unittest

import six

import conan.tools.qbs.qbs as qbs

from conans.client import tools
from conans.errors import ConanException
from conans.test.utils.mocks import MockConanfile, MockSettings
from conans.test.utils.test_files import temp_folder


class RunnerMock(object):
    def __init__(self, return_ok=True, output=None):
        self.command_called = None
        self.return_ok = return_ok
        self.output = output

    def __call__(self, command, output, win_bash=False, subsystem=None):
        self.command_called = command
        self.win_bash = win_bash
        self.subsystem = subsystem
        if self.output and output and hasattr(output, 'write'):
            output.write(self.output)
        return 0 if self.return_ok else 1


class QbsTest(unittest.TestCase):
    def test_generating_config_command_line(self):
        name = 'default'
        flag_dict = {
            'modules.cpp.cxxFlags': ['-frtti', '-fexceptions'],
            'modules.cpp.ldFlags': '--defsym=hello_world',
            'products.App.myIntProperty': 13,
            'products.App.myBoolProperty': True
        }
        expected_config_line = [
            'config:%s' % name,
        ]
        for key, value in flag_dict.items():
            if type(value) is bool:
                if value:
                    value = 'true'
                else:
                    value = 'false'
            expected_config_line.append('%s:%s' % (key, value))
        self.assertEqual(
            qbs._configuration_dict_to_commandlist(name, flag_dict),
            expected_config_line)

    def test_construct_build_helper_without_project_file(self):
        conanfile = MockConanfile(
            MockSettings({'os': 'Linux', 'compiler': 'gcc'}))
        conanfile.folders.set_base_source('.')
        build_helper = qbs.Qbs(conanfile)
        self.assertEqual(build_helper.jobs, tools.cpu_count())
        self.assertEqual(build_helper._project_file, conanfile.source_folder)

    def test_construct_build_helper_with_project_file(self):
        conanfile = MockConanfile(
            MockSettings({'os': 'Linux', 'compiler': 'gcc'}))
        # just asume that the test is called from repo root
        profile_file_path = temp_folder()
        build_helper = qbs.Qbs(conanfile, project_file=profile_file_path)
        self.assertEqual(build_helper._project_file, profile_file_path)

    def test_construct_build_helper_with_wrong_project_file_path(self):
        conanfile = MockConanfile(
            MockSettings({'os': 'Linux', 'compiler': 'gcc'}))
        with self.assertRaises(ConanException):
            qbs.Qbs(conanfile, project_file='random/file/path')

    def test_add_configuration(self):
        conanfile = MockConanfile(
            MockSettings({'os': 'Linux', 'compiler': 'gcc'}))
        conanfile.folders.set_base_source('.')
        build_helper = qbs.Qbs(conanfile)
        configurations = {
            'debug':  {'products.MyLib.specialFlags': ['-frtti',
                                                       '-fexceptions']},
            'release': {'products.MyLib.specialFlags': ['-fno-exceptions',
                                                        '-fno-rtti']}
        }
        for name, config in configurations.items():
            build_helper.add_configuration(name, config)
        self.assertEqual(build_helper._configuration, configurations)

    def test_build(self):
        conanfile = MockConanfile(
            MockSettings({'os': 'Linux', 'compiler': 'gcc'}),
            runner=RunnerMock())
        conanfile.folders.set_base_source('.')
        conanfile.folders.set_base_build('.')
        build_helper = qbs.Qbs(conanfile)

        build_helper.build()
        self.assertEqual(
            conanfile.runner.command_called,
            ('qbs build --no-install --build-directory %s '
             '--file %s --jobs %s profile:%s') % (
                conanfile.build_folder, build_helper._project_file,
                build_helper.jobs, build_helper.profile))

        build_helper.build(products=['app1', 'app2', 'lib'])
        self.assertEqual(
            conanfile.runner.command_called,
            ('qbs build --no-install --build-directory %s '
             '--file %s --products app1,app2,lib --jobs %s profile:%s') % (
                conanfile.build_folder, build_helper._project_file,
                build_helper.jobs, build_helper.profile))

    def test_build_all(self):
        conanfile = MockConanfile(
            MockSettings({'os': 'Linux', 'compiler': 'gcc'}),
            runner=RunnerMock())
        conanfile.folders.set_base_source('.')
        conanfile.folders.set_base_build('.')
        build_helper = qbs.Qbs(conanfile)

        build_helper.build_all()
        self.assertEqual(
            conanfile.runner.command_called,
            ('qbs build --no-install --build-directory %s '
             '--file %s --all-products --jobs %s profile:%s') % (
                conanfile.build_folder, build_helper._project_file,
                build_helper.jobs, build_helper.profile))

    @unittest.skipIf(six.PY2, "Order of qbs output is defined only for PY3")
    def test_build_with_custom_configuration(self):
        conanfile = MockConanfile(
            MockSettings({'os': 'Linux', 'compiler': 'gcc'}),
            runner=RunnerMock())
        conanfile.folders.set_base_source('.')
        conanfile.folders.set_base_build('.')
        build_helper = qbs.Qbs(conanfile)
        config_name = 'debug'
        config_values = {
            'product.App.boolProperty': True,
            'product.App.intProperty': 1337,
            'product.App.stringProperty': 'Hello World',
            'product.App.stringListProperty': ['Hello', 'World']
        }
        build_helper.add_configuration(config_name, config_values)
        build_helper.build()
        self.assertEqual(
            conanfile.runner.command_called,
            ('qbs build --no-install --build-directory %s '
             '--file %s --jobs %s profile:%s '
             'config:%s %s:%s %s:%s %s:%s %s:%s') % (
                conanfile.build_folder, build_helper._project_file,
                build_helper.jobs, build_helper.profile,
                config_name,
                'product.App.boolProperty',
                'true',
                'product.App.intProperty',
                config_values['product.App.intProperty'],
                'product.App.stringProperty',
                config_values['product.App.stringProperty'],
                'product.App.stringListProperty',
                config_values['product.App.stringListProperty']))

    def test_install(self):
        conanfile = MockConanfile(
            MockSettings({'os': 'Linux', 'compiler': 'gcc'}),
            runner=RunnerMock())
        conanfile.folders.set_base_source('.')
        conanfile.folders.set_base_package("pkg")
        build_helper = qbs.Qbs(conanfile)

        build_helper.install()
        self.assertEqual(
            conanfile.runner.command_called,
            ('qbs install --no-build --install-root %s '
             '--file %s') % (
                conanfile.package_folder, build_helper._project_file))

    def test_install_with_custom_configuration(self):
        conanfile = MockConanfile(
            MockSettings({'os': 'Linux', 'compiler': 'gcc'}),
            runner=RunnerMock())
        conanfile.folders.set_base_source('.')
        conanfile.folders.set_base_package("pkg")
        build_helper = qbs.Qbs(conanfile)
        config_name = 'debug'
        config_values = {
            'product.App.boolProperty': True,
            'product.App.intProperty': 1337,
            'product.App.stringProperty': 'Hello World',
            'product.App.stringListProperty': ['Hello', 'World']
        }
        build_helper.add_configuration(config_name, config_values)

        build_helper.install()
        self.assertEqual(
            conanfile.runner.command_called,
            ('qbs install --no-build --install-root %s '
             '--file %s config:%s') % (
                conanfile.package_folder, build_helper._project_file,
                config_name))



import json
import textwrap

from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient


def test_basic_validate_build_test():

    t = TestClient()
    conanfile = textwrap.dedent("""
    from conan import ConanFile
    from conans.errors import ConanInvalidConfiguration

    class myConan(ConanFile):
        name = "foo"
        version = "1.0"
        settings = "os", "arch", "compiler"

        def validate_build(self):
            if self.settings.compiler == "gcc":
                raise ConanInvalidConfiguration("This doesn't build in GCC")

        def package_id(self):
            del self.info.settings.compiler
    """)

    settings_gcc = "-s compiler=gcc -s compiler.libcxx=libstdc++11 -s compiler.version=11"
    settings_clang = "-s compiler=clang -s compiler.libcxx=libc++ -s compiler.version=8"

    t.save({"conanfile.py": conanfile})
    t.run(f"create . {settings_gcc}", assert_error=True)

    assert "foo/1.0: Cannot build for this configuration: This doesn't build in GCC" in t.out

    t.run(f"create . {settings_clang}")

    # Now with GCC again, but now we have the binary, we don't need to build, so it doesn't fail
    t.run(f"create . {settings_gcc} --build missing")
    assert "foo/1.0: Already installed!" in t.out

    # But if I force the build... it will fail
    t.run(f"create . {settings_gcc} ", assert_error=True)
    assert "foo/1.0: Cannot build for this configuration: This doesn't build in GCC" in t.out

    # What happens with a conan info?
    t.run(f"info foo/1.0@ {settings_gcc} --json=myjson")
    myjson = json.loads(t.load("myjson"))
    assert myjson[0]["invalid_build"] is True
    assert myjson[0]["invalid_build_reason"] == "This doesn't build in GCC"

    t.run(f"info foo/1.0@ {settings_clang} --json=myjson")
    myjson = json.loads(t.load("myjson"))
    assert myjson[0]["invalid_build"] is False
    assert "invalid_build_reason" not in myjson[0]


def test_with_options_validate_build_test():
    t = TestClient()
    conanfile = textwrap.dedent("""
    from conan import ConanFile
    from conans.errors import ConanInvalidConfiguration

    class myConan(ConanFile):
        name = "foo"
        version = "1.0"
        options = {"my_option": [True, False]}
        default_options = {"my_option": True}

        def validate_build(self):
            if not self.options.my_option:
                raise ConanInvalidConfiguration("This doesn't build with False option")

    """)
    t.save({"conanfile.py": conanfile})
    t.run("export .")
    consumer = GenConanfile().with_require("foo/1.0").with_name("consumer").with_version("1.0")
    t.save({"consumer.py": consumer})
    t.run("create consumer.py --build missing -o foo/*:my_option=False", assert_error=True)
    assert "foo/1.0: Cannot build for this configuration: This doesn't build " \
           "with False option" in t.out

    t.run("create consumer.py --build missing -o foo/*:my_option=True")



import textwrap

from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient


def test_dependencies_visit():
    client = TestClient()
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . openssl/0.1@")
    client.run("create . openssl/0.2@")
    client.save({"conanfile.py": GenConanfile().with_requires("openssl/0.2")})
    client.run("create . cmake/0.1@")
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class Pkg(ConanFile):
            requires = "openssl/0.1"
            build_requires = "cmake/0.1"

            def generate(self):
                dep = self.dependencies["openssl"]
                self.output.info("DefRef: {}!!!".format(repr(dep.ref)))
                self.output.info("DefPRef: {}!!!".format(repr(dep.pref)))
                dep = self.dependencies.build["openssl"]
                self.output.info("DefRefBuild: {}!!!".format(repr(dep.ref)))
                self.output.info("DefPRefBuild: {}!!!".format(repr(dep.pref)))
                for r, d in self.dependencies.build.items():
                    self.output.info("DIRECTBUILD {}: {}".format(r.direct, d))
        """)
    client.save({"conanfile.py": conanfile})
    client.run("install . -s:b os=Windows")  # Use 2 contexts
    assert "DefRef: openssl/0.1#f3367e0e7d170aa12abccb175fee5f97!!!" in client.out
    assert "DefPRef: openssl/0.1#f3367e0e7d170aa12abccb175fee5f97:"\
           "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9#"\
           "83c38d3b4e5f1b8450434436eec31b00!!!" in client.out

    assert "DefRefBuild: openssl/0.2#f3367e0e7d170aa12abccb175fee5f97!!!" in client.out
    assert "DefPRefBuild: openssl/0.2#f3367e0e7d170aa12abccb175fee5f97:" \
           "5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9#" \
           "83c38d3b4e5f1b8450434436eec31b00!!!" in client.out

    assert "conanfile.py: DIRECTBUILD True: cmake/0.1" in client.out
    assert "conanfile.py: DIRECTBUILD False: openssl/0.2" in client.out


def test_dependencies_visit_settings_options():
    client = TestClient()
    client.save({"conanfile.py": GenConanfile().with_settings("os").
                with_option("shared", [True, False]).with_default_option("shared", False)})
    client.run("create . openssl/0.1@ -s os=Linux")
    client.save({"conanfile.py": GenConanfile().with_requires("openssl/0.1")})
    client.run("create . pkg/0.1@  -s os=Linux")
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class Pkg(ConanFile):
            requires = "pkg/0.1"

            def generate(self):
                dep = self.dependencies["openssl"]
                self.output.info("SETTINGS: {}!".format(dep.settings.os))
                self.output.info("OPTIONS: shared={}!".format(dep.options.shared))
        """)
    client.save({"conanfile.py": conanfile})
    client.run("install . -s os=Linux")
    assert "conanfile.py: SETTINGS: Linux!" in client.out
    assert "conanfile.py: OPTIONS: shared=False!" in client.out


def test_dependencies_visit_build_requires_profile():
    """ At validate() time, in Conan 1.X, the build-requires are not available yet, because first
    the binary package_id is computed, then the build-requires are resolved.
    It is necessary to avoid in Conan 1.X the caching of the ConanFile.dependencies, because
    at generate() time it will have all the graph info, including build-requires
    """
    # https://github.com/conan-io/conan/issues/10304
    client = TestClient()
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . cmake/0.1@")
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class Pkg(ConanFile):

            def validate(self):
                self.output.info("VALIDATE DEPS: {}!!!".format(len(self.dependencies.items())))

            def generate(self):
                for req, dep in self.dependencies.items():
                    self.output.info("GENERATE REQUIRE: {}!!!".format(dep.ref))
                dep = self.dependencies.build["cmake"]
                self.output.info("GENERATE CMAKE: {}!!!".format(dep.ref))
        """)
    client.save({"conanfile.py": conanfile,
                 "profile": "[build_requires]\ncmake/0.1"})
    client.run("install . -pr:b=default -pr:h=profile --build")  # Use 2 contexts
    # Validate time, build-requires not available yet
    assert "conanfile.py: VALIDATE DEPS: 0!!!" in client.out
    # generate time, build-requires already available
    assert "conanfile.py: GENERATE REQUIRE: cmake/0.1!!!" in client.out
    assert "conanfile.py: GENERATE CMAKE: cmake/0.1!!!" in client.out


def test_dependency_interface():
    """
    Quick test for the ConanFileInterface exposed
    """
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conans import ConanFile
        class Pkg(ConanFile):
            name = "dep"
            version = "1.0"
            homepage = "myhome"
            url = "myurl"
            license = "MIT"
        """)
    user = textwrap.dedent("""
        from conan import ConanFile
        class User(ConanFile):
            requires = "dep/1.0"
            def generate(self):
                self.output.info("HOME: {}".format(self.dependencies["dep"].homepage))
                self.output.info("URL: {}".format(self.dependencies["dep"].url))
                self.output.info("LICENSE: {}".format(self.dependencies["dep"].license))
                self.output.info("RECIPE: {}".format(self.dependencies["dep"].recipe_folder))
                self.output.info("CONANDATA: {}".format(self.dependencies["dep"].conan_data))

            """)
    c.save({"dep/conanfile.py": conanfile,
            "dep/conandata.yml": "",
            "user/conanfile.py": user})
    c.run("create dep")
    c.run("install user")
    assert "conanfile.py: HOME: myhome" in c.out
    assert "conanfile.py: URL: myurl" in c.out
    assert "conanfile.py: LICENSE: MIT" in c.out
    assert "conanfile.py: RECIPE:" in c.out
    assert "conanfile.py: CONANDATA: {}" in c.out



import textwrap

import pytest

from conans.test.assets.genconanfile import GenConanfile
from conans.test.utils.tools import TestClient
from conans.tools import save


@pytest.fixture
def client():
    client = TestClient()
    conanfile = str(GenConanfile())
    conanfile += """
    def package_info(self):
        self.buildenv_info.define("Foo", "MyVar!")
        self.runenv_info.define("runFoo", "Value!")
        self.buildenv_info.append("Hello", "MyHelloValue!")
    """
    client.save({"conanfile.py": conanfile})
    client.run("create . foo/1.0@")
    save(client.cache.new_config_path, "tools.env.virtualenv:auto_use=True")
    return client


def test_virtualenv_object_access(client):
    conanfile = textwrap.dedent("""
    import os
    from conans import ConanFile
    from conan.tools.env import VirtualBuildEnv, VirtualRunEnv

    class ConanFileToolsTest(ConanFile):
        requires = "foo/1.0"

        def build(self):
          build_env = VirtualBuildEnv(self).vars()
          run_env = VirtualRunEnv(self).vars()
          self.output.warn("Foo: *{}*".format(build_env["Foo"]))
          self.output.warn("runFoo: *{}*".format(run_env["runFoo"]))
          self.output.warn("Hello: *{}*".format(build_env["Hello"]))

          with build_env.apply():
            with run_env.apply():
              self.output.warn("Applied Foo: *{}*".format(os.getenv("Foo", "")))
              self.output.warn("Applied Hello: *{}*".format(os.getenv("Hello", "")))
              self.output.warn("Applied runFoo: *{}*".format(os.getenv("runFoo", "")))
    """)

    profile = textwrap.dedent("""
            [buildenv]
            Foo+=MyFooValue
        """)

    client.save({"conanfile.py": conanfile, "profile": profile})

    client.run("create . app/1.0@ --profile=profile")
    assert "Foo: *MyVar! MyFooValue*"
    assert "runFoo:* Value!*"
    assert "Hello:* MyHelloValue!*"

    assert "Applied Foo: *MyVar! MyFooValue*"
    assert "Applied runFoo: **"
    assert "Applied Hello: * MyHelloValue!*"



import textwrap
import unittest

from conans.test.utils.tools import TestClient


class MarkDownGeneratorTest(unittest.TestCase):
    def test_cmake_find_filename(self):
        conanfile = textwrap.dedent("""
                    from conans import ConanFile

                    class HelloConan(ConanFile):
                        def package_info(self):
                            self.cpp_info.set_property("cmake_file_name", "FooBar")
                            self.cpp_info.set_property("cmake_target_name", "foobar")
                            self.cpp_info.set_property("pkg_config_name", "foobar_cfg")
                    """)
        client = TestClient()
        client.save({"conanfile.py": conanfile})
        client.run("create . bar/0.1.0@user/testing")
        client.run("install bar/0.1.0@user/testing -g markdown")
        content = client.load("bar.md")

        self.assertIn("find_package(FooBar)", content)
        self.assertIn("target_link_libraries(${PROJECT_NAME} foobar)", content)

    def test_cmake_find_filename_with_namespace(self):
        conanfile = textwrap.dedent("""
                    from conans import ConanFile

                    class HelloConan(ConanFile):
                        def package_info(self):
                            self.cpp_info.set_property("cmake_file_name", "FooBar")
                            self.cpp_info.set_property("cmake_target_name", "foobar::foobar")
                            self.cpp_info.set_property("pkg_config_name", "foobar_cfg")
                    """)
        client = TestClient()
        client.save({"conanfile.py": conanfile})
        client.run("create . bar/0.1.0@user/testing")
        client.run("install bar/0.1.0@user/testing -g markdown")
        content = client.load("bar.md")

        self.assertIn("find_package(FooBar)", content)
        self.assertIn("target_link_libraries(${PROJECT_NAME} foobar::foobar)", content)

    def test_with_build_modules(self):
        conanfile = textwrap.dedent("""
                    import os
                    from conans import ConanFile

                    class HelloConan(ConanFile):
                        exports_sources = 'bm.cmake'
                        def package(self):
                            self.copy('bm.cmake', dst='lib/cmake')

                        def package_info(self):
                            self.cpp_info.set_property("cmake_file_name", "FooBar")
                            self.cpp_info.set_property("cmake_target_name", "foobar")
                            self.cpp_info.set_property("pkg_config_name", "foobar_cfg")
                            self.cpp_info.set_property('cmake_build_modules', ['lib/cmake/bm.cmake'])
                    """)
        client = TestClient()
        client.save({"conanfile.py": conanfile,
                     "bm.cmake": "Content of build_module" })
        client.run("create . bar/0.1.0@user/testing")
        client.run("install bar/0.1.0@user/testing -g markdown")
        content = client.load("bar.md")

        self.assertIn("#### lib/cmake/bm.cmake", content)
        self.assertIn("Content of build_module", content)

    def test_no_components(self):
        conanfile = textwrap.dedent("""
                    import os
                    from conans import ConanFile

                    class HelloConan(ConanFile):
                        def package_info(self):
                            self.cpp_info.set_property("cmake_target_name", "foobar")
                    """)
        client = TestClient()
        client.save({"conanfile.py": conanfile})
        client.run("create . bar/0.1.0@user/testing")
        client.run("install bar/0.1.0@user/testing -g markdown")
        content = client.load("bar.md")

        self.assertNotIn("Or link just one of its components", content)
        self.assertNotIn("Declared components", content)

    def test_with_components(self):
        conanfile = textwrap.dedent("""
                    import os
                    from conans import ConanFile

                    class HelloConan(ConanFile):
                        def package_info(self):
                            self.cpp_info.set_property("cmake_target_name", "foobar")
                            self.cpp_info.components["component1"].set_property("cmake_target_name", "foobar::component_name")
                    """)
        client = TestClient()
        client.save({"conanfile.py": conanfile})
        client.run("create . bar/0.1.0@user/testing")
        client.run("install bar/0.1.0@user/testing -g markdown")
        content = client.load("bar.md")

        self.assertIn("target_link_libraries(${PROJECT_NAME} foobar::component_name)", content)
        self.assertIn("* CMake target name: ``foobar::component_name``", content)

    def test_with_components_and_target_namespace(self):
        conanfile = textwrap.dedent("""
                    import os
                    from conans import ConanFile

                    class HelloConan(ConanFile):
                        def package_info(self):
                            self.cpp_info.set_property("cmake_target_name", "namespace::name")
                            self.cpp_info.components["component1"].set_property("cmake_target_name", "namespace::component_name")
                    """)
        client = TestClient()
        client.save({"conanfile.py": conanfile})
        client.run("create . bar/0.1.0@user/testing")
        client.run("install bar/0.1.0@user/testing -g markdown")
        content = client.load("bar.md")

        self.assertIn("target_link_libraries(${PROJECT_NAME} namespace::name)", content)
        self.assertIn("* CMake target name: ``namespace::component_name``", content)

    def test_c_project(self):
        conanfile = textwrap.dedent("""
                    from conans import ConanFile
                    class HelloConan(ConanFile):
                        settings = "os", "arch", "compiler", "build_type"
                        def configure(self):
                            del self.settings.compiler.libcxx
                            del self.settings.compiler.cppstd
                        def package_info(self):
                            self.cpp_info.set_property("cmake_file_name", "FooBar")
                            self.cpp_info.set_property("cmake_target_name", "foobar")
                            self.cpp_info.set_property("pkg_config_name", "foobar_cfg")
                    """)
        client = TestClient()
        client.save({"conanfile.py": conanfile})
        client.run("create . bar/0.1.0@user/testing")
        client.run("install bar/0.1.0@user/testing -g markdown")
        content = client.load("bar.md")
        self.assertIn("main.c", content)
        self.assertIn("project(bar_project C)", content)

    def test_with_sys_requirements(self):
        conanfile = textwrap.dedent("""
                    import os
                    from conans import ConanFile

                    class HelloConan(ConanFile):
                        def package_info(self):
                            self.cpp_info.components["component1"].system_libs = ["system_lib"]
                    """)
        client = TestClient()
        client.save({"conanfile.py": conanfile})
        client.run("create . bar/0.1.0@user/testing")
        client.run("install bar/0.1.0@user/testing -g markdown")
        assert "Generator markdown created bar.md" in client.out



import os
import unittest

from conans.model.ref import ConanFileReference, PackageReference
from conans.util.env_reader import get_env
from conans.test.utils.tools import TestClient, TestServer, NO_SETTINGS_PACKAGE_ID, GenConanfile


class CorruptedPackagesTest(unittest.TestCase):
    """
    Simulate a connection failure or file corruption in the server with missing files for a
    package and make sure the search, install are possible. Check re-upload is always possible
    even if the package in the server is not accessible
    """

    def setUp(self):
        revisions_enabled = get_env("TESTING_REVISIONS_ENABLED", False)
        self.server = TestServer([("*/*@*/*", "*")], [("*/*@*/*", "*")])
        self.client = TestClient(servers={"default": self.server})
        self.client.save({"conanfile.py": GenConanfile()})
        self.client.run("create . Pkg/0.1@user/testing")
        self.client.run("upload * --all --confirm -r default")
        # Check files are uploded in this order: conan_package.tgz, conaninfo.txt, conanmanifest.txt
        order1 = str(self.client.out).find("Uploading conan_package.tgz")
        order2 = str(self.client.out).find("Uploading conaninfo.txt", order1)
        order3 = str(self.client.out).find("Uploading conanmanifest.txt", order2)
        self.assertTrue(order1 < order2 < order3)
        rrev = "f3367e0e7d170aa12abccb175fee5f97" if revisions_enabled else "0"
        pref_str = "Pkg/0.1@user/testing#%s" % rrev
        prev = "83c38d3b4e5f1b8450434436eec31b00" if revisions_enabled else "0"
        self.pref = pref = PackageReference(ConanFileReference.loads(pref_str),
                                            NO_SETTINGS_PACKAGE_ID, prev)
        self.manifest_path = self.server.server_store.get_package_file_path(pref,
                                                                            "conanmanifest.txt")
        self.info_path = self.server.server_store.get_package_file_path(pref, "conaninfo.txt")
        self.tgz_path = self.server.server_store.get_package_file_path(pref, "conan_package.tgz")

    def _assert_all_package_files_in_server(self):
        self.assertTrue(os.path.exists(self.manifest_path))
        self.assertTrue(os.path.exists(self.info_path))
        self.assertTrue(os.path.exists(self.tgz_path))

    def test_info_manifest_missing(self):
        os.unlink(self.info_path)
        os.unlink(self.manifest_path)
        # Try search
        self.client.run("search Pkg/0.1@user/testing -r default")
        self.assertIn("There are no packages for reference 'Pkg/0.1@user/testing', "
                      "but package recipe found", self.client.out)
        # Try fresh install
        self.client.run("remove * -f")
        self.client.run("install Pkg/0.1@user/testing", assert_error=True)
        self.assertIn("Pkg/0.1@user/testing:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Missing",
                      self.client.out)
        # Try upload of fresh package
        self.client.run("create . Pkg/0.1@user/testing")
        self.client.run("upload * --all --confirm -r default")
        self._assert_all_package_files_in_server()

    def test_manifest_missing(self):
        os.unlink(self.manifest_path)
        # Try search
        self.client.run("search Pkg/0.1@user/testing -r default")
        self.assertIn("Package_ID: 5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9", self.client.out)
        # Try fresh install
        self.client.run("remove * -f")
        self.client.run("install Pkg/0.1@user/testing", assert_error=True)
        self.assertIn("ERROR: Binary package not found", self.client.out)
        self.assertIn(NO_SETTINGS_PACKAGE_ID, self.client.out)
        # Try upload of fresh package
        self.client.run("create . Pkg/0.1@user/testing")
        self.client.run("upload * --all --confirm")
        self._assert_all_package_files_in_server()

    def test_tgz_info_missing(self):
        os.unlink(self.tgz_path)
        os.unlink(self.info_path)
        # Try search
        self.client.run("search Pkg/0.1@user/testing -r default")
        self.assertIn("There are no packages for reference 'Pkg/0.1@user/testing', "
                      "but package recipe found", self.client.out)
        # Try fresh install
        self.client.run("remove * -f")
        self.client.run("install Pkg/0.1@user/testing", assert_error=True)
        self.assertIn("Pkg/0.1@user/testing:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Missing",
                      self.client.out)
        # Try upload of fresh package
        self.client.run("create . Pkg/0.1@user/testing")
        self.client.run("upload * --all --confirm")
        self.assertIn("Uploading conan_package.tgz", self.client.out)
        self.assertIn("Uploading conaninfo.txt", self.client.out)
        self._assert_all_package_files_in_server()

    def test_tgz_missing(self):
        os.unlink(self.tgz_path)
        # Try search
        self.client.run("search Pkg/0.1@user/testing -r default")
        # Try fresh install
        self.client.run("remove * -f")
        self.client.run("install Pkg/0.1@user/testing", assert_error=True)
        self.assertIn("ERROR: Binary package not found", self.client.out)
        # Try upload of fresh package
        self.client.run("create . Pkg/0.1@user/testing")
        self.client.run("upload * --all --confirm")
        self.assertIn("Uploading conan_package.tgz", self.client.out)
        self._assert_all_package_files_in_server()

    def test_tgz_manifest_missing(self):
        os.unlink(self.tgz_path)
        os.unlink(self.manifest_path)
        # Try search
        self.client.run("search Pkg/0.1@user/testing -r default")
        self.assertIn("Package_ID: 5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9", self.client.out)
        # Try fresh install
        self.client.run("remove * -f")
        self.client.run("install Pkg/0.1@user/testing", assert_error=True)
        self.assertIn("ERROR: Binary package not found", self.client.out)
        # Try upload of fresh package
        self.client.run("create . Pkg/0.1@user/testing")
        self.client.run("upload * --all --confirm")
        self._assert_all_package_files_in_server()

    def test_tgz_manifest_info_missing(self):
        os.unlink(self.tgz_path)
        os.unlink(self.manifest_path)
        os.unlink(self.info_path)
        # Try search
        self.client.run("search Pkg/0.1@user/testing -r default")
        self.assertIn("There are no packages for reference 'Pkg/0.1@user/testing', "
                      "but package recipe found", self.client.out)
        # Try fresh install
        self.client.run("remove * -f")
        self.client.run("install Pkg/0.1@user/testing", assert_error=True)
        self.assertIn("Pkg/0.1@user/testing:5ab84d6acfe1f23c4fae0ab88f26e3a396351ac9 - Missing",
                      self.client.out)
        # Try upload of fresh package
        self.client.run("create . Pkg/0.1@user/testing")
        self.client.run("upload * --all --confirm")
        self._assert_all_package_files_in_server()



import os
import time
import traceback
from conans.client.cmd.uploader import CmdUpload, _UploadCollecter
from multiprocessing.pool import ThreadPool
from conans.util.progress_bar import left_justify_message
from conans.tools import cpu_count
from conans.model.ref import ConanFileReference, PackageReference, check_valid_ref
from conans.util.log import logger
from conans.errors import ConanException, NotFoundException
from conans.util.env_reader import get_env


class CmdUploadDeb(CmdUpload):
    def __init__(self, cache, user_io, remote_manager, loader, hook_manager, graph_manager):
        super().__init__(cache, user_io, remote_manager, loader, hook_manager)
        self._graph_manager = graph_manager
    
    
    def upload(self, reference_or_pattern, remotes, upload_recorder, package_id=None,
               all_packages=None, confirm=False, retry=None, retry_wait=None, integrity_check=False,
               policy=None, query=None, parallel_upload=False):
        t1 = time.time()
        
        collecter = _UploadCollecter(self._cache, self._user_io, self._output, self._loader)
        refs_by_remote = collecter.collect(package_id, reference_or_pattern, confirm, remotes,
                                           all_packages, query)
        
        if parallel_upload:
            self._user_io.disable_input()
        self._upload_thread_pool = ThreadPool(
            cpu_count() if parallel_upload else 1)

        for remote, refs in refs_by_remote.items():
            self._output.info("Uploading to remote '{}':".format(remote.name))

            def upload_ref(ref_conanfile_prefs):
                _ref, _conanfile, _prefs = ref_conanfile_prefs
                try:
                    self._upload_ref(_conanfile, _ref, _prefs, retry, retry_wait,
                                     integrity_check, policy, remote, upload_recorder, remotes)
                except BaseException as base_exception:
                    base_trace = traceback.format_exc()
                    self._exceptions_list.append((base_exception, _ref, base_trace, remote))

            self._upload_thread_pool.map(upload_ref,
                                         [(ref, conanfile, prefs) for (ref, conanfile, prefs) in
                                          refs])

        self._upload_thread_pool.close()
        self._upload_thread_pool.join()

        if len(self._exceptions_list) > 0:
            for exc, ref, trace, remote in self._exceptions_list:
                t = "recipe" if isinstance(ref, ConanFileReference) else "package"
                msg = "%s: Upload %s to '%s' failed: %s\n" % (str(ref), t, remote.name, str(exc))
                if get_env("CONAN_VERBOSE_TRACEBACK", False):
                    msg += trace
                self._output.error(msg)
            raise ConanException("Errors uploading some packages")

        logger.debug("UPLOAD: Time manager upload: %f" % (time.time() - t1))


    def _upload_ref(self, conanfile, ref, prefs, retry, retry_wait, integrity_check, policy,
                    recipe_remote, upload_recorder, remotes):
        """ Uploads the recipes and binaries identified by ref
        """
        assert (ref.revision is not None), "Cannot upload a recipe without RREV"
        conanfile_path = self._cache.package_layout(ref).conanfile()
        # FIXME: I think it makes no sense to specify a remote to "pre_upload"
        # FIXME: because the recipe can have one and the package a different one
        self._hook_manager.execute("pre_upload", conanfile_path=conanfile_path,
                                   reference=ref, remote=recipe_remote)
        msg = "\rUploading %s to remote '%s'" % (str(ref), recipe_remote.name)
        self._output.info(left_justify_message(msg))
       
        # Now the binaries
        if prefs:
            total = len(prefs)
            p_remote = recipe_remote

            def upload_package_index(index_pref):
                index, pref = index_pref
                try:
                    up_msg = "\rUploading package %d/%d: %s to '%s'" % (index + 1, total,
                                                                        str(pref.id),
                                                                        p_remote.name)
                    self._output.info(left_justify_message(up_msg))
                    self._upload_deb(pref, retry, retry_wait, integrity_check, policy, p_remote)
                    # upload_recorder.add_package(pref, p_remote.name, p_remote.url)
                except BaseException as pkg_exc:
                    trace = traceback.format_exc()
                    return pkg_exc, pref, trace, p_remote

            def upload_package_callback(ret):
                package_exceptions = [r for r in ret if r is not None]
                self._exceptions_list.extend(package_exceptions)
                if not package_exceptions:
                    # FIXME: I think it makes no sense to specify a remote to "post_upload"
                    # FIXME: because the recipe can have one and the package a different one
                    self._hook_manager.execute("post_upload", conanfile_path=conanfile_path,
                                               reference=ref, remote=recipe_remote)

            # This doesn't wait for the packages to end, so the function returns
            # and the "pool entry" for the recipe is released
            self._upload_thread_pool.map_async(upload_package_index,
                                               [(index, pref) for index, pref
                                                in enumerate(prefs)],
                                               callback=upload_package_callback)
        else:
            # FIXME: I think it makes no sense to specify a remote to "post_upload"
            # FIXME: because the recipe can have one and the package a different one
            self._hook_manager.execute("post_upload", conanfile_path=conanfile_path, reference=ref,
                                       remote=recipe_remote)

    def _upload_deb(self, pref, retry=None, retry_wait=None, integrity_check=False,
                        policy=None, p_remote=None):

        assert (pref.revision is not None), "Cannot upload a package without PREV"
        assert (pref.ref.revision is not None), "Cannot upload a package without RREV"

        pkg_layout = self._cache.package_layout(pref.ref)
        conanfile_path = pkg_layout.conanfile()
        
        install_fold = pkg_layout.build(pref)
        conanfile = self._graph_manager.load_consumer_conanfile(conanfile_path, install_fold,
                                                                   deps_info_required=True)
        deb_folder = pkg_layout.deb(pref)
        deb_file = os.path.join(deb_folder, conanfile.name + ".deb")
        
        if os.path.exists(deb_file):
            # upload deb file
            upload_deb_cmd = 'curl -u "'+ conanfile.user +'" -H "Content-Type: multipart/form-data" --data-binary "@' \
                                + deb_file + '" "' + conanfile.apt_sever + '"'
            conanfile.run(upload_deb_cmd)
        else:
            print("Warning: deb file is not exist")

        return pref




# coding=utf-8

import os
import platform
import tarfile
import unittest

import pytest

from conans.client.tools.files import chdir
from conans.model.manifest import gather_files
from conans.test.utils.test_files import temp_folder
from conans.util.files import tar_extract, gzopen_without_timestamps, save


class TarExtractTest(unittest.TestCase):

    def setUp(self):
        self.tmp_folder = temp_folder()
        with chdir(self.tmp_folder):
            # Create a couple of files
            ori_files_dir = os.path.join(self.tmp_folder, "ori")
            file1 = os.path.join(ori_files_dir, "file1")
            file2 = os.path.join(ori_files_dir, "folder", "file2")
            save(file1, "")
            save(file2, "")

            # Create a tar.gz file with the above files
            self.tgz_file = os.path.join(self.tmp_folder, "file.tar.gz")
            with open(self.tgz_file, "wb") as tgz_handle:
                tgz = gzopen_without_timestamps("name", mode="w", fileobj=tgz_handle)

                files, _ = gather_files(ori_files_dir)
                for filename, abs_path in files.items():
                    info = tarfile.TarInfo(name=filename)
                    with open(file1, 'rb') as file_handler:
                        tgz.addfile(tarinfo=info, fileobj=file_handler)
                tgz.close()

    @pytest.mark.skipif(platform.system() != "Linux", reason="Requires Linux")
    def test_link_folder(self):
        # If there is a linked folder in the current directory that matches one file in the tar.
        # https://github.com/conan-io/conan/issues/4959

        # Once unpackaged, this is the content of the destination directory
        def check_files(destination_dir):
            d = sorted(os.listdir(destination_dir))
            self.assertListEqual(d, ["file1", "folder"])
            d_folder = os.listdir(os.path.join(destination_dir, "folder"))
            self.assertEqual(d_folder, ["file2"])

        working_dir = temp_folder()
        with chdir(working_dir):
            # Unpack and check
            destination_dir = os.path.join(self.tmp_folder, "dest")
            with open(self.tgz_file, 'rb') as file_handler:
                tar_extract(file_handler, destination_dir)
            check_files(destination_dir)

            # Unpack and check (now we have a symlinked local folder)
            os.symlink(temp_folder(), "folder")
            destination_dir = os.path.join(self.tmp_folder, "dest2")
            with open(self.tgz_file, 'rb') as file_handler:
                tar_extract(file_handler, destination_dir)
            check_files(destination_dir)



import os
import textwrap

from conans.test.utils.tools import TestClient


def test_file_tools():

    conanfile = textwrap.dedent("""

    from conan import ConanFile
    from conan.tools.files import rmdir, mkdir

    class pkg(ConanFile):

        def layout(self):
            self.folders.generators = "gen"

        def generate(self):
            mkdir(self, "folder1")
            mkdir(self, "folder2")
            rmdir(self, "folder2")

    """)

    client = TestClient()
    client.save({"conanfile.py": conanfile})
    client.run("install . ")
    assert os.path.exists(os.path.join(client.current_folder, "gen", "folder1"))
    assert not os.path.exists(os.path.join(client.current_folder, "gen", "folder2"))
